﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace ELTE.Windows.HelloWorld
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            Loaded += new RoutedEventHandler(Startup);
        }

        private void Startup(Object sender, RoutedEventArgs e)
        {
            _Button.Opacity = 0; // a gomb kezdő átlátszósága

            DoubleAnimation opacityAnimation = new DoubleAnimation();
            opacityAnimation.From = 0;
            opacityAnimation.To = 1;
            opacityAnimation.Duration = TimeSpan.FromSeconds(2);
            opacityAnimation.BeginTime = TimeSpan.FromSeconds(3);

            _Button.BeginAnimation(Button.OpacityProperty, opacityAnimation); // animáció indítása

            /* ugyanez jelenettel:
            Storyboard opacityStoryBoard = new Storyboard();
            opacityStoryBoard.Children.Add(opacityAnimation);
            Storyboard.SetTarget(opacityStoryBoard, _Button); // célobjektum beállítás
            Storyboard.SetTargetProperty(opacityStoryBoard, new PropertyPath(Button.OpacityProperty)); // céltulajdonság beállítás
            opacityStoryBoard.BeginTime = TimeSpan.FromSeconds(3); // 3 másodperces késleltetés a kezdésre
            opacityStoryBoard.Begin(); // jelenet elindítása
            */
        }

        private void ButtonClick(Object sender, RoutedEventArgs e)
        {
            (sender as Button).Content = "Thanks!"; // küldő objektum lekérdezése

            RotateTransform rotateTransform = new RotateTransform(); // forgatási transzformáció
            rotateTransform.CenterX = _HelloWorldLabel.Width / 2;
            rotateTransform.CenterY = _HelloWorldLabel.Height / 2;
            rotateTransform.Angle = 0; // kezdetben az eredeti állapot

            _HelloWorldLabel.RenderTransform = rotateTransform; // a transzformációt  beállítjuk az objektumra

            DoubleAnimation rotateAnimation = new DoubleAnimation();
            rotateAnimation.From = 0;
            rotateAnimation.To = 360;
            rotateAnimation.Duration = TimeSpan.FromSeconds(3);
            rotateAnimation.Completed += new EventHandler(RotateAnimationCompleted); // eseménykezelő futtatása az animáció után

            rotateTransform.BeginAnimation(RotateTransform.AngleProperty, rotateAnimation, HandoffBehavior.SnapshotAndReplace); // animáció indítása

            /* ugyanez jelenettel:
            Storyboard rotateStoryBoard = new Storyboard(); // jelenet objektum
            rotateStoryBoard.Children.Add(rotateAnimation); // animáció felvétele a jelenethez
            Storyboard.SetTarget(rotateStoryBoard, rotateTransform); // célobjektum beállítás, ami most egy transzformációs objektum lesz
            Storyboard.SetTargetProperty(rotateStoryBoard, new PropertyPath(RotateTransform.AngleProperty)); // céltulajdonság beállítás
            rotateStoryBoard.Completed += new EventHandler(RotateAnimation_Completed); // eseménykezelő futtatás befejezéskor
            rotateStoryBoard.Begin();
            */
        }

        private void RotateAnimationCompleted(Object sender, EventArgs e)
        {
            ScaleTransform scaleTransform = new ScaleTransform(); // átméretezési transzformáció
            scaleTransform.CenterX = _HelloWorldLabel.Width / 2;
            scaleTransform.CenterY = _HelloWorldLabel.Height / 2;
            scaleTransform.ScaleX = 1; // kezdetben az eredeti állapot
            scaleTransform.ScaleY = 1;

            _HelloWorldLabel.RenderTransform = scaleTransform; // a transzformációt objektumot beállítjuk az objekrumra
            // _HelloWorldLabel.RenderTransformOrigin = new Point(0.5, 0.5); // a transzformáció kiindulási pontja

            DoubleAnimation scaleXAnimation = new DoubleAnimation();
            scaleXAnimation.From = 1;
            scaleXAnimation.To = 2;
            scaleXAnimation.Duration = TimeSpan.FromMilliseconds(500);

            DoubleAnimation scaleYAnimation = new DoubleAnimation();
            scaleYAnimation.From = 1;
            scaleYAnimation.To = 3;
            scaleYAnimation.Duration = TimeSpan.FromMilliseconds(500);

            ColorAnimation colorAnimation = new ColorAnimation(); // szín animáció
            colorAnimation.From = Colors.Black;
            colorAnimation.To = Colors.Blue;
            colorAnimation.Duration = TimeSpan.FromMilliseconds(500);

            Storyboard labelStoryBoard = new Storyboard();
            labelStoryBoard.Children.Add(scaleXAnimation);
            labelStoryBoard.Children.Add(scaleYAnimation);
            labelStoryBoard.Children.Add(colorAnimation);

            Storyboard.SetTarget(scaleXAnimation, _HelloWorldLabel); // az animációt a címkére alkalmazzuk
            Storyboard.SetTarget(scaleYAnimation, _HelloWorldLabel);
            Storyboard.SetTarget(colorAnimation, _HelloWorldLabel);
            Storyboard.SetTargetProperty(scaleXAnimation, new PropertyPath("(Control.RenderTransform).(ScaleTransform.ScaleX)")); // összetett tulajdonság szöveges megadása
            Storyboard.SetTargetProperty(scaleYAnimation, new PropertyPath("(Control.RenderTransform).(ScaleTransform.ScaleY)"));
            Storyboard.SetTargetProperty(colorAnimation, new PropertyPath("(Control.Foreground).(SolidColorBrush.Color)"));

            labelStoryBoard.AutoReverse = true;
            labelStoryBoard.RepeatBehavior = RepeatBehavior.Forever; // jelenet ismétlése
            labelStoryBoard.Begin();
        }
    }
}
