﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace ELTE.Windows.SimpleBallGame
{
    public partial class MainWindow : Window
    {
        private Thickness _BallStartPosition; // pozíciók, pontosabban határoló téglalapok
        private Thickness _BallCurrentPosition;
        private Thickness _BallNextPosition;
        private Thickness _PadStartPosition;
        private Thickness _PadCurrentPosition;

        private ThicknessAnimation _BallAnimation; // animációk
        private ThicknessAnimation _PadAnimation;

        private DateTime _GameStartTime; // játék kezdőideje
        private Boolean _Playing; // tart-e a játék
        private Double _Speed; // játék sebessége

        public MainWindow()
        {
            InitializeComponent();

            _BallStartPosition = _EllipseBall.Margin;
            _BallCurrentPosition = _BallStartPosition;
            _BallNextPosition = new Thickness();
            _PadStartPosition = _RectanglePad.Margin;
            _PadCurrentPosition = _PadStartPosition;
            _Playing = false;

            KeyDown += new KeyEventHandler(Navigate);
            _EllipseBall.LayoutUpdated += new EventHandler(BallLayoutUpdated);
        }

        private void NewGame(Object sender, RoutedEventArgs e)
        {
            StartGame();
        }

        private void Exit(Object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void Navigate(Object sender, KeyEventArgs e)
        {
            _PadCurrentPosition = _RectanglePad.Margin;
            Int32 movement = 0;
            if (e.Key == Key.Left) // megállapítjuk, milyen gombot ütött le
                if (_RectanglePad.Margin.Left > 0) // csak akkor mozgunk, ha még van hova
                    movement = -100;
            if (e.Key == Key.Right)
                if (_RectanglePad.Margin.Right > 0)
                    movement = 100;

            AnimatePad(movement);
        }

        private void BallLayoutUpdated(Object sender, EventArgs e)
        {
            if (_Playing) // csak akkor, ha tart a játék
            {
                _BallCurrentPosition = _EllipseBall.Margin;

                // ütőnek ütközik balról:
                if ((_EllipseBall.Margin.Top + _EllipseBall.Height >= _RectanglePad.Margin.Top) && _EllipseBall.Margin.Left >= (_RectanglePad.Margin.Left - 30) && _EllipseBall.Margin.Left <= (_RectanglePad.Margin.Left + 30))
                {
                    _Speed *= 1.05; // sebességnövelés
                    _BallNextPosition.Top = 0; // új cél a tető
                    _BallNextPosition.Left = _BallCurrentPosition.Left - 200; // és balra haladás
                    AnimateBall();
                }
                // ütőnek ütközik jobbról: 
                else if ((_EllipseBall.Margin.Top + _EllipseBall.Height >= _RectanglePad.Margin.Top) && _EllipseBall.Margin.Left >= (_RectanglePad.Margin.Left + 30) && _EllipseBall.Margin.Left <= (_RectanglePad.Margin.Left + 110))
                {
                    _Speed *= 1.05;
                    _BallNextPosition.Top = 0;
                    _BallNextPosition.Left = _BallCurrentPosition.Left + 200;
                    AnimateBall();
                }
                else if (_EllipseBall.Margin.Top <= 25) // tetőnek ütközik
                {
                    _BallNextPosition.Top = Height;
                    AnimateBall();
                }
                else if (_EllipseBall.Margin.Left <= 0) // bal falnak ütközik
                {
                    _BallNextPosition.Left = Width;
                    AnimateBall();
                }
                else if (_EllipseBall.Margin.Left >= Width - 40) // jobb falnak ütközik
                {
                    _BallNextPosition.Left = 0;
                    AnimateBall();
                }
                else if (_EllipseBall.Margin.Top > _RectanglePad.Margin.Top) // túlment az ütőn
                {
                    _BallNextPosition.Top = 0;
                    MessageBox.Show("Összesen " + (DateTime.Now - _GameStartTime).ToString().Substring(0, 8) + " ideig tartott a játék.", "Vége a játéknak!");
                    StopGame();
                }
            }
        }

        /// <summary>
        /// Játék indítása.
        /// </summary>
        private void StartGame()
        {
            Random random = new Random();

            _GameStartTime = DateTime.Now;
            _BallCurrentPosition = _BallStartPosition; // a labdát a kezdőpontra helyezzük
            _BallNextPosition.Left = random.Next(- 1, 1) * Width; // a kezdőirány véletlenszerű
            _BallNextPosition.Top = random.Next(-1, 1) * Height;
            _Speed = 1;
            _Playing = true;

            AnimateBall();
        }

        /// <summary>
        /// Játék leállítása.
        /// </summary>
        private void StopGame()
        {
            _Playing = false;
        }

        /// <summary>
        /// Labda animálása.
        /// </summary>
        private void AnimateBall()
        {
            _BallAnimation = new ThicknessAnimation(); // új animáció létrehozása
            _BallAnimation.From = _BallCurrentPosition; // a labda jelenlegi pozíciójából
            _BallAnimation.To = _BallNextPosition; // az új pozícióba
            _BallAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(5)); // animáció ideje
            _BallAnimation.SpeedRatio = _Speed / BallTravelDistance(); // az animáció sebessége függ a játék sebességétől, illetve a labda utazási távolságától

            _EllipseBall.BeginAnimation(Rectangle.MarginProperty, _BallAnimation, HandoffBehavior.SnapshotAndReplace); 
            // animáció lejátszása az adott objektumon és tulajdonságon
            // ha már fut animáció, akkor annak leállítása
        }

        /// <summary>
        /// Ütő animálása.
        /// </summary>
        /// <param name="x">A mozgatás mértéke pontban.</param>
        private void AnimatePad(Int32 x)
        {
            _PadAnimation = new ThicknessAnimation();
            _PadAnimation.From = _PadCurrentPosition;
            _PadAnimation.To = new Thickness(_RectanglePad.Margin.Left + x, _RectanglePad.Margin.Top, _RectanglePad.Margin.Right - x, _RectanglePad.Margin.Bottom);
            _PadAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(100));

            _RectanglePad.BeginAnimation(Rectangle.MarginProperty, _PadAnimation, HandoffBehavior.SnapshotAndReplace);
        }

        /// <summary>
        /// Labda utazási távolságának kiszámítása.
        /// </summary>
        /// <returns></returns>
        private Double BallTravelDistance()
        {
            return (Math.Sqrt(Math.Pow(Math.Abs(_BallNextPosition.Left - _BallCurrentPosition.Left), 2) + Math.Pow(Math.Abs(_BallNextPosition.Top - _BallCurrentPosition.Top), 2)));
        }
    }
}
