﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Microsoft.Win32;

namespace ELTE.Windows.SlideGame
{
    public partial class MainWindow : Window
    {
        BitmapImage _Image; // megjelenített kép
        Rectangle[,] _Rectangles; // a játék téglalapjai
        OpenFileDialog _OpenFileDialog; // fájl megnyitó dialógus

        public MainWindow()
        {
            InitializeComponent();

            _Rectangles = new Rectangle[4, 4];
        }

        /// <summary>
        /// Új kirakó létrehozása.
        /// </summary>
        private void CreatePuzzle()
        {
            // korábbi elemek törlése, ha voltak:
            _Grid.Children.Clear();

            // négyzetek feltöltése képekkel:
            for (Int32 x = 0; x < _Rectangles.GetLength(0); x++)
                for (Int32 y = 0; y < _Rectangles.GetLength(1); y++)
                {
                    if (x != 0 || y != 0) // az első négyzet üresen marad
                    {
                        ImageBrush imageBrush = new ImageBrush(); // képfestő ecset
                        imageBrush.Stretch = Stretch.UniformToFill; // kép széthúzása
                        imageBrush.ImageSource = _Image; // forráskép
                        imageBrush.Viewport = new Rect(0, 0, 1.0, 1.0); // nézet megadása

                        imageBrush.Viewbox = new Rect(1.0 * x / _Rectangles.GetLength(0), 1.0 * y / _Rectangles.GetLength(1), 1.0 / _Rectangles.GetLength(0), 1.0 / _Rectangles.GetLength(1)); // kivásághoz megfelelő kijelölőnégyzet
                        imageBrush.ViewboxUnits = BrushMappingMode.RelativeToBoundingBox; // amely a mérethez képest relatív
                        imageBrush.TileMode = TileMode.None; // hatráron túli kitöltés módja

                        Rectangle imageRectangle = new Rectangle();
                        imageRectangle.Fill = imageBrush; // kitöltés a kivágott képpel
                        imageRectangle.Margin = new Thickness(0); // a határt 0 vastagra vesszük
                        imageRectangle.HorizontalAlignment = HorizontalAlignment.Stretch; // a téglalap nyújtható lesz
                        imageRectangle.VerticalAlignment = VerticalAlignment.Stretch;
                        imageRectangle.MouseDown += new MouseButtonEventHandler(Step); // esemény a téglalaphoz

                        _Rectangles[x, y] = imageRectangle;
                        _Grid.Children.Add(_Rectangles[x, y]); // elem felvétele a rácsra
                    }
                }

            // üres négyzet betöltése:
            Rectangle blankRectangle = new Rectangle();
            blankRectangle.Fill = new SolidColorBrush(Colors.White); // kitöltés fehér lesz
            blankRectangle.Margin = new Thickness(0); // a határt 0 vastagra vesszük
            blankRectangle.HorizontalAlignment = HorizontalAlignment.Stretch; // a téglalap nyújtható lesz
            blankRectangle.VerticalAlignment = VerticalAlignment.Stretch;

            _Rectangles[0, 0] = blankRectangle;
            _Grid.Children.Add(_Rectangles[0, 0]);

            RandomizeTiles();
        }

        /// <summary>
        /// Elemek véletlenített betöltése.
        /// </summary>
        private void RandomizeTiles()
        {
            Random random = new Random();

            Int32[] orderArray = new Int32[_Rectangles.Length]; // permutáláshoz használt tömb
            Int32 source, dest;

            for (Int32 i = 0; i < orderArray.Length; i++)
                orderArray[i] = i;

            for (Int32 i = 0; i < orderArray.Length; i++)
            {               
                source = random.Next(orderArray.Length);
                dest = random.Next(orderArray.Length);

                Int32 temp = orderArray[dest]; // permutálás végrehajtása
                orderArray[dest] = orderArray[source];
                orderArray[source] = temp;
            }

            for (Int32 i = 0; i < orderArray.Length; i++)
            {
                // függőségi tulajdonságok beállítása:
                _Rectangles[i / _Rectangles.GetLength(1), i % _Rectangles.GetLength(1)].SetValue(Grid.ColumnProperty, orderArray[i] % _Rectangles.GetLength(1));
                _Rectangles[i / _Rectangles.GetLength(1), i % _Rectangles.GetLength(1)].SetValue(Grid.RowProperty, orderArray[i] / _Rectangles.GetLength(1));
            }
        }

        /// <summary>
        /// Rejtvény megoldottságának ellenőrzése.
        /// </summary>
        /// <returns>Igaz, ha minden képrész a helyén van.</returns>
        private Boolean IsPuzzleSolved()
        {
            Boolean solved = true;
            for (Int32 x = 0; x < _Rectangles.GetLength(0) && solved; x++)
                for (Int32 y = 0; y < _Rectangles.GetLength(1) && solved; y++)
                {
                    solved = (_Rectangles[x, y].GetValue(Grid.ColumnProperty).Equals(x) && _Rectangles[x, y].GetValue(Grid.RowProperty).Equals(y));
                    // a helyzet ellenőrzését a függőségi tulajdonsággal végezzük
                }

            return solved;
        }

        /// <summary>
        /// Négyzeten történő egérkattintás eseménykezelője.
        /// </summary>
        /// <param name="sender">Eseményküldő objektum.</param>
        /// <param name="e">Eseményparaméterek.</param>
        private void Step(Object sender, MouseButtonEventArgs e)
        {
            // lekérjük az aktuális és az üres négyzetet:
            Rectangle currentRectangle = sender as Rectangle;
            Rectangle blankRectangle = _Rectangles[0, 0];

            // függőségi tualjdonságok használata:
            Int32 currentCol = Convert.ToInt32(currentRectangle.GetValue(Grid.ColumnProperty));
            Int32 currentRow = Convert.ToInt32(currentRectangle.GetValue(Grid.RowProperty));
            Int32 blankCol = Convert.ToInt32(blankRectangle.GetValue(Grid.ColumnProperty));
            Int32 blankRow = Convert.ToInt32(blankRectangle.GetValue(Grid.RowProperty));

            if (Math.Abs(currentCol - blankCol) + Math.Abs(currentRow - blankRow) == 1) // ha a két elem szomszédos
            {
                currentRectangle.SetValue(Grid.ColumnProperty, blankCol); // elemek cseréje
                currentRectangle.SetValue(Grid.RowProperty, blankRow);

                blankRectangle.SetValue(Grid.ColumnProperty, currentCol);
                blankRectangle.SetValue(Grid.RowProperty, currentRow);

                if (IsPuzzleSolved()) // ha vége a játéknak
                {
                    MessageBox.Show("Gratulálok, kiraktad!", "Játék vége", MessageBoxButton.OK, MessageBoxImage.Information);
                    RandomizeTiles();
                }

            }
        }

        private void LoadImage(object sender, RoutedEventArgs e)
        {
            _OpenFileDialog = new Microsoft.Win32.OpenFileDialog(); // fájl betöltő dialógusablak
            _OpenFileDialog.Title = "Kép betöltése";
            _OpenFileDialog.Filter = "Képfájlok (*.BMP;*.JPG;*.GIF;*.PNG)|*.BMP;*.JPG;*.GIF;*.PNG|Minden fájl (*.*)|*.*";
            _OpenFileDialog.Multiselect = false;

            if (_OpenFileDialog.ShowDialog() == true)
            {
                try
                {
                    _Image = new BitmapImage(new Uri(_OpenFileDialog.FileName, UriKind.RelativeOrAbsolute));
                    Width = _Image.Width;
                    Height = _Image.Height + 25;
                    CreatePuzzle();
                }
                catch
                {
                    MessageBox.Show("Fájl megnyitása sikertelen!", "Hiba!", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }
    }
}
